@@ -288,7 +288,7 @@ static int generate_data(short* from,
         sample /= speed_factor;
       }
 
-      encode_sample((short) sample, dsp->format,
+      encode_sample((short) (sample * dsp->volume), dsp->format,
           &result[(i * dsp->channels + j) * dsp->samplesize / 8]);
 
     }
@@ -389,63 +389,75 @@ int sndfile_get_samples(const char* filename,
 #endif /* WITH_SNDFILE */
 
 /*
- * allocates and initializes dsp->tickdata{0,1,2}
- * and initializes dsp->td{0,1,2}_size
+ * allocates and initializes dsp->frames
+ * and initializes dsp->number_of_frames
  * according to dsp->soundname
  *
  * returns 0 on success, -1 otherwise
  */
 static int init_sample(dsp_t* dsp) {
-  short *tmp_buf; /* temporary buffer for generation of different sounds */
-  int i;
-
-  short* samples;
-  int frames = 0;
-  int rate = 0;
-  int channels = 0;
-
-  int attack = 0;
-  
   if (!strcmp(dsp->soundname, "<default>")) {
-    samples = tickdata;
-    frames = (signed int) sizeof(tickdata) / sizeof(short);
-    rate = 44100;
-    channels = 1;
+    dsp->frames = (short*) malloc(sizeof(tickdata));
+    memcpy(dsp->frames, tickdata, sizeof(tickdata));
+    dsp->number_of_frames = (signed int) sizeof(tickdata) / sizeof(short);
+    dsp->rate_in = 44100;
+    dsp->channels_in = 1;
   } else if (!strcmp(dsp->soundname, "<sine>")) {
-    frames = generate_sine(44100, SIN_FREQ, SIN_DUR, FADE_DUR, &samples);
-    rate = 44100;
-    channels = 1;
+    dsp->number_of_frames =
+      generate_sine(44100, SIN_FREQ, SIN_DUR, FADE_DUR, &dsp->frames);
+    dsp->rate_in = 44100;
+    dsp->channels_in = 1;
   } else {
 #ifdef WITH_SNDFILE
-    frames = sndfile_get_samples(dsp->soundname, &samples, &rate, &channels);
+    dsp->number_of_frames = sndfile_get_samples(dsp->soundname, &dsp->frames,
+	&dsp->rate_in, &dsp->channels_in);
 #else
     fprintf(stderr, "Warning: Unhandled sample name case: \"%s\".\n",
 	    dsp->soundname);
 #endif /* WITH_SNDFILE */
   }
 
-  if (frames == -1)
+  if (dsp->number_of_frames == -1)
     return -1;
+
+  return 0;
+}
+  
+/*
+ * allocates and initializes dsp->tickdata{0,1,2}
+ * and initializes dsp->td{0,1,2}_size
+ * according to dsp->frames and dsp->number_of_frames
+ *
+ * returns 0 on success, -1 otherwise
+ */
+static int prepare_buffers(dsp_t* dsp) {
+  short *tmp_buf; /* temporary buffer for generation of different sounds */
+  int i;
+
+  int attack = 0;
   
   /* generate single ticks */
-  if ((dsp->td0_size = generate_data(samples, frames, rate, channels,
-                                     dsp, &dsp->tickdata0)) == -1)
+  if ((dsp->td0_size = generate_data(dsp->frames, dsp->number_of_frames,
+	  dsp->rate_in, dsp->channels_in, dsp, &dsp->tickdata0)) == -1)
   {
     return -1;
   }
 
   /* generate first tick */
-  if ((dsp->td1_size = generate_data(samples, frames, rate * 2, channels,
-                                     dsp, &dsp->tickdata1)) == -1)
+  if ((dsp->td1_size = generate_data(dsp->frames, dsp->number_of_frames,
+	  dsp->rate_in * 2, dsp->channels_in, dsp, &dsp->tickdata1)) == -1)
   {
     return -1;
   }
 
   /* attack padding for accents */
-  while (attack < frames && abs(samples[attack]) < SHRT_MAX / 20)
+  while (attack <
+      dsp->number_of_frames && abs(dsp->frames[attack]) < SHRT_MAX / 20)
+  {
     attack++;
+  }
 
-  if (attack < frames / 3) {
+  if (attack < dsp->number_of_frames / 3) {
     unsigned char* newdata;
     int offset;
     
@@ -469,24 +481,20 @@ static int init_sample(dsp_t* dsp) {
   }
 
   /* generate secondary ticks */
-  if (!(tmp_buf = (short*) malloc(frames * sizeof(short)))) {
+  if (!(tmp_buf = (short*) malloc(dsp->number_of_frames * sizeof(short)))) {
     perror("malloc");
     return -1;
   }
-  for (i = 0; i < frames; i++) {
-    tmp_buf[i] = samples[i] / 2;
+  for (i = 0; i < dsp->number_of_frames; i++) {
+    tmp_buf[i] = dsp->frames[i] / 2;
   }
-  if ((dsp->td2_size = generate_data(tmp_buf, frames, rate, channels,
-                                    dsp, &dsp->tickdata2)) == -1)
+  if ((dsp->td2_size = generate_data(tmp_buf, dsp->number_of_frames,
+	  dsp->rate_in, dsp->channels_in, dsp, &dsp->tickdata2)) == -1)
   {
     return -1;
   }
   free(tmp_buf);
 
-  if (strcmp(dsp->soundname, "<default>")) { /* other than default */
-    free(samples);
-  }
-
   return 0;
 }
 
@@ -500,7 +508,6 @@ int dsp_open(dsp_t* dsp) {
   unsigned int format_index;
   int requested_format = DEFAULT_FORMAT;
   audio_buf_info info;
-  int mask;
   
   dsp->fragmentsize = 0x7fff0008; /* at least request fragment size 2^8=256 */
                                     /* = minimum recommended size */
@@ -598,21 +605,6 @@ int dsp_open(dsp_t* dsp) {
     g_print("dsp_open: Total number of fragments in DSP buffer = %d.\n",
 	    dsp->fragstotal);
   
-  /* mixer check */
-  if (ioctl(dsp->dspfd, SOUND_MIXER_READ_DEVMASK, &mask) == -1)
-    dsp->mixer_supported = 0;
-  else
-    if (mask & SOUND_MIXER_PCM)
-      dsp->mixer_supported = 0;
-    else
-      dsp->mixer_supported = 1;
-  if (debug && debug_todo) {
-    if (dsp->mixer_supported)
-      fprintf(stderr, "PCM channel mixer supported.\n");
-    else
-      fprintf(stderr, "PCM channel mixer not supported.\n");
-  }
-  
   debug_todo = 0;
   return 0;
 }
@@ -652,13 +644,23 @@ int dsp_init(dsp_t* dsp)
   if (dsp_open(dsp) == -1)
     return -1;
   
+  dsp->frames = NULL;
+  dsp->silence = NULL;
+  dsp->tickdata0 = NULL;
+  dsp->tickdata1 = NULL;
+  dsp->tickdata2 = NULL;
+  
   /* silence */
   generate_data(&silencelevel, 1, dsp->rate, 1, dsp, &dsp->silence);
 
-  /* allocate and initialize dsp->tickdata{0,1,2} */
+  /* allocate and initialize dsp->frames */
   if (init_sample(dsp) == -1) {
     return -1;
   }
+  /* allocate and initialize dsp->tickdata{0,1,2} */
+  if (prepare_buffers(dsp) == -1) {
+    return -1;
+  }
 
   dsp->cyclepos = 0; /* init */
   dsp->tickpos = 0;
@@ -692,6 +694,10 @@ void dsp_deinit(dsp_t* dsp)
     g_free(dsp->silence);
     dsp->silence = NULL;
   }
+  if (dsp->frames) {
+    g_free(dsp->frames);
+    dsp->frames = NULL;
+  }
 }
 
 /*
@@ -784,88 +790,30 @@ gboolean dsp_feed(dsp_t* dsp)
 }
 
 /*
- * Gets PCM value in mixer setting
+ * Gets mixer setting
  *
- * returns value from 0 to 100 on success, -1 otherwise
+ * returns value from 0.0 to 1.0 on success
  */
-int dsp_get_volume(dsp_t* dsp)
+double dsp_get_volume(dsp_t* dsp)
 {
-  int vol, r, l;
-  int close_on_exit;
-  
-  if (dsp->dspfd < 0) {
-    if (dsp_open(dsp) == -1) {
-      dsp_close(dsp);
-      return -1;
-    } else {
-      close_on_exit = 1;
-    }
-  } else {
-    close_on_exit = 0;
-  }
-
-  if (!dsp->mixer_supported) {
-    if (close_on_exit)
-      dsp_close(dsp);
-    return -1;
-  }
-  
-  if (ioctl(dsp->dspfd, MIXER_READ(SOUND_MIXER_PCM), &vol) == -1) {
-    if (close_on_exit)
-      dsp_close(dsp);
-    return -1;
-  }
-  
-  l = vol & 0xff;
-  r = (vol & 0xff00) >> 8;
-
-  if (debug >= 2)
-    g_print("dsp_get_volume: vol=%d, l=%d, r=%d\n", vol, l, r);
-
-  if (close_on_exit)
-    dsp_close(dsp);
-
-  return (r+l)/2;
+  return dsp->volume;
 }
 
 /*
- * Sets PCM value in mixer setting
- * accepts a value from 0 to 100
+ * Sets mixer setting
+ *
+ * accepts a value from 0.0 to 1.0
  */
-void dsp_set_volume(dsp_t* dsp, int vol)
+void dsp_set_volume(dsp_t* dsp, double volume)
 {
-  int tvol;
-  int close_on_exit;
-  
-  if (dsp->dspfd < 0) {
-    if (dsp_open(dsp) == -1) {
-      dsp_close(dsp);
-      return;
-    } else {
-      close_on_exit = 1;
-    }
-  } else {
-    close_on_exit = 0;
+  dsp->volume = volume;
+  if (dsp->running) {
+    free(dsp->tickdata0);
+    free(dsp->tickdata1);
+    free(dsp->tickdata2);
+    
+    prepare_buffers(dsp);
   }
-
-  if (!dsp->mixer_supported) {
-    if (close_on_exit)
-      dsp_close(dsp);
-    return;
-  }
-  
-  tvol = (vol << 8) + vol;
-
-  if (debug)
-    g_print("dsp_set_volume: Setting mixer value: 0x%04X\n", tvol);
-
-  ioctl(dsp->dspfd, MIXER_WRITE(SOUND_MIXER_PCM), &tvol);
-
-  if ((dsp->realvol = dsp_get_volume(dsp)) == -1)
-    dsp->realvol = VOLUME_MAX;
-
-  if (close_on_exit)
-    dsp_close(dsp);
 }
 
 /*
@@ -880,9 +828,7 @@ void dsp_main_loop(dsp_t* dsp) {
   while (repeat_flag) {
     message_type_t message_type;
     void* message;
-    int new_volume = -1;        /* set a volume */
     int get_volume = 0;         /* flag */
-    int get_volume_change = 0;  /* flag */
     void* reply = NULL;
     
     while ((message_type = comm_server_try_get_query(dsp->inter_thread_comm,
@@ -931,43 +877,34 @@ void dsp_main_loop(dsp_t* dsp) {
 	  dsp->sync_flag = 0;
 	  break;
 	case MESSAGE_TYPE_SET_VOLUME:
-	  new_volume = *((int*) message);
+	  dsp_set_volume(dsp, *((double*) message));
 	  free(message);
 	  break;
 	case MESSAGE_TYPE_GET_VOLUME:
 	  get_volume = 1;
 	  break;
-	case MESSAGE_TYPE_GET_VOLUME_CHANGE:
-	  get_volume_change = 1;
-	  break;
 	default:
 	  fprintf(stderr, "Warning: Unhandled message type in audio thread.\n");
       }
     }
-    if (new_volume >= 0) {
-      dsp_set_volume(dsp, new_volume);
-    }
-    if (get_volume || get_volume_change) {
-      int volume = dsp_get_volume(dsp);
+
+    if (get_volume) {
+      double volume = dsp_get_volume(dsp);
       
-      if (get_volume || (get_volume_change && volume != dsp->realvol)) {
-	if (get_volume_change) {
-	  dsp->realvol = volume;
-	}
-	if (!(reply = (int*) malloc (sizeof(int)))) {
-	  printf("Error: "
-		 "Insufficient memory to allocate for volume reply.\n");
-	  exit(1);
-	}
-	*((int*)reply) = volume;
-	if (*((int*)reply) != -1) {
-	  comm_server_send_response(dsp->inter_thread_comm,
-				    MESSAGE_TYPE_RESPONSE_VOLUME, reply);
-	} else {
-	  free(reply);
-	}
+      if (!(reply = (double*) malloc (sizeof(double)))) {
+	printf("Error: "
+	       "Insufficient memory to allocate for volume reply.\n");
+	exit(1);
+      }
+      *((double*)reply) = volume;
+      if (*((double*)reply) != -1) {
+	comm_server_send_response(dsp->inter_thread_comm,
+				  MESSAGE_TYPE_RESPONSE_VOLUME, reply);
+      } else {
+	free(reply);
       }
     }
+
     if (dsp->running) { /* metronome running */
       dsp_feed(dsp);
     }
