@@ -97,14 +97,15 @@ int gui_get_meter(metro_t* metro) {
  */
 static void set_volume_cb(metro_t* metro)
 {
-  int* volume = (int*) malloc(sizeof(int));
+  double* volume = (double*) malloc(sizeof(double));
 
   if (volume == NULL) {
     printf("Error: Insufficient memory for allocating volume message.\n");
     exit(1);
   }
-  *volume = GTK_ADJUSTMENT(metro->volume_adjustment)->value;
-  comm_client_query(metro->inter_thread_comm, MESSAGE_TYPE_SET_VOLUME, volume);
+  *volume = GTK_ADJUSTMENT(metro->volume_adjustment)->value / 100.0;
+  comm_client_query(metro->inter_thread_comm, MESSAGE_TYPE_SET_VOLUME,
+      volume);
 }
 
 /*
@@ -190,17 +191,12 @@ static void set_state(metro_t* metro, int state) {
 static gint handle_comm(metro_t* metro) {
   message_type_t message_type;
   void* body = NULL;
-  int newvolume = -1;
 
   while ((message_type = comm_client_try_get_reply(metro->inter_thread_comm,
 	                                           &body)) !=
          MESSAGE_TYPE_NO_MESSAGE)
   {
     switch (message_type) {
-      case MESSAGE_TYPE_RESPONSE_VOLUME:
-	newvolume = *((int*) body);
-	free(body);
-	break;
       case MESSAGE_TYPE_RESPONSE_SYNC:
 	visualtick_sync(metro, *((unsigned int*) body));
 	free(body);
@@ -215,18 +211,6 @@ static gint handle_comm(metro_t* metro) {
     }
   }
 
-  if (newvolume >= 0) {
-    gtk_adjustment_set_value(GTK_ADJUSTMENT(metro->volume_adjustment),
-			     newvolume);
-    if (!metro->volume_initialized) {
-      g_signal_connect_swapped(G_OBJECT(metro->volume_adjustment),
-			       "value_changed",
-			       G_CALLBACK(set_volume_cb),
-			       metro);
-      metro->volume_initialized = 1;
-    }
-  }
-
   return TRUE;
 }
 
@@ -299,9 +283,6 @@ static gint timeout_callback(metro_t* metro) {
     quit_cb(metro, 0, NULL);
   }
 
-  comm_client_query(metro->inter_thread_comm,
-                    MESSAGE_TYPE_GET_VOLUME_CHANGE, NULL);
-
   return TRUE; /* call it again */
 }
 
@@ -595,6 +576,55 @@ static int new_speed(metro_t* metro) {
 static void delete_speed(metro_t* metro _U_) {
 }
 
+/*
+ * option system callback for choosing the volume
+ *
+ * returns 0 on success, -1 otherwise
+ */
+static int set_volume(metro_t* metro, const char* volume)
+{
+  if (!metro || !metro->volume_adjustment || !volume)
+    return -1;
+  
+  gtk_adjustment_set_value(GTK_ADJUSTMENT(metro->volume_adjustment),
+                           strtod_c(volume));
+
+  return 0;
+}
+
+/*
+ * option system callback for spotting the volume
+ *
+ * if called with metro == NULL, returns NULL
+ */
+static const char* get_volume(metro_t* metro) {
+  if (metro == NULL) {
+    return NULL;
+  } else {
+    return dtostr_c(gtk_adjustment_get_value(
+	                GTK_ADJUSTMENT(metro->volume_adjustment)), 1);
+  }
+}
+
+/*
+ * option system callback for initializing volume option
+ */
+static int new_volume(metro_t* metro) {
+  if (metro->volume_adjustment) {
+    gtk_adjustment_set_value(GTK_ADJUSTMENT(metro->volume_adjustment),
+                             DEFAULT_VOLUME);
+    return 0;
+  } else {
+    return -1;
+  }
+}
+
+/*
+ * option system callback for destroying volume option
+ */
+static void delete_volume(metro_t* metro _U_) {
+}
+
 /*
  * Sets the specified meter (type int)
  */
@@ -1261,6 +1291,14 @@ metro_t* metro_new(void) {
 		  (option_get_t) get_speed,
 		  (void*) metro);
   
+  option_register(&metro->options->option_list,
+                  "Volume",
+		  (option_new_t) new_volume,
+		  (option_delete_t) delete_volume,
+		  (option_set_t) set_volume,
+		  (option_get_t) get_volume,
+		  (void*) metro);
+  
   label = gtk_label_new (_("Volume:"));
   gtk_table_attach(GTK_TABLE(table), label,
                    0, 1, 1, 2,
@@ -1268,9 +1306,13 @@ metro_t* metro_new(void) {
   gtk_misc_set_alignment(GTK_MISC(label), 1, 0.5);
   gtk_widget_show (label);
 	  
-  metro->volume_adjustment = gtk_adjustment_new (0.0, VOLUME_MIN, VOLUME_MAX,
-                                    5.0, 25.0, 0.0);
-  /* we set up the callback later, only when the mixer sent an initial volume */
+  metro->volume_adjustment = gtk_adjustment_new(DEFAULT_VOLUME,
+      VOLUME_MIN, VOLUME_MAX, 5.0, 25.0, 0.0);
+  g_signal_connect_swapped(G_OBJECT(metro->volume_adjustment),
+			   "value_changed",
+			   G_CALLBACK(set_volume_cb),
+			   metro);
+  set_volume_cb(metro);
   
   scale = gtk_hscale_new (GTK_ADJUSTMENT(metro->volume_adjustment));
   gtk_range_set_update_policy (GTK_RANGE (scale), GTK_UPDATE_CONTINUOUS);
@@ -1361,7 +1403,7 @@ metro_t* metro_new(void) {
                    G_CALLBACK(set_meter_cb),
 	           metro);
   
-  adjustment = gtk_adjustment_new(5, 1, MAX_METER, 1, 4, 0);
+  adjustment = gtk_adjustment_new(5, 5, MAX_METER, 1, 4, 0);
   metro->meter_spin_button =
     gtk_spin_button_new(GTK_ADJUSTMENT(adjustment), 1, 0);
   gtk_widget_set_sensitive(metro->meter_spin_button, FALSE);
@@ -1485,8 +1527,6 @@ metro_t* metro_new(void) {
   /* read rc file */
   option_restore_all(metro->options->option_list);
 
-  comm_client_query(metro->inter_thread_comm, MESSAGE_TYPE_GET_VOLUME, NULL);
-
   /* set up additional handlers */
   gtk_timeout_add(TIMER_DELAY, (GtkFunction) timeout_callback, metro);
   gtk_timeout_add(VISUAL_DELAY * 1000, (GtkFunction) handle_comm, metro);
